## Copyright (c) 2005-2006, Gillmer J. Derge.

## This file is part of Civilization IV Alerts mod.
##
## Civilization IV Alerts mod is free software; you can redistribute
## it and/or modify it under the terms of the GNU General Public
## License as published by the Free Software Foundation; either
## version 2 of the License, or (at your option) any later version.
##
## Civilization IV Alerts mod is distributed in the hope that it will
## be useful, but WITHOUT ANY WARRANTY; without even the implied
## warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
## See the GNU General Public License for more details.
##
## You should have received a copy of the GNU General Public License
## along with Civilization IV Alerts mod; if not, write to the Free
## Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
## 02110-1301 USA
	
__version__ = "$Revision: 1.3 $"
# $Source: /usr/local/cvsroot/Civ4lerts/src/main/python/CvCustomEventManager.py,v $


import CvEventManager
import os
import CvPath
import zConfigParser



##-------------------------------------------------------------------

class CvEventManagerControlFile(CvEventManager.CvEventManager, object):

	"""Extends the standard event manager by adding support for multiple
	handlers for each event.
	
	Methods exist for both adding and removing event handlers.  A set method 
	also exists to override the default handlers.  Clients should not depend 
	on event handlers being called in a particular order.
	
	This approach works best with mods that have implemented the design
	pattern suggested on Apolyton by dsplaisted.
	
	http://apolyton.net/forums/showthread.php?threadid=142916
	
	The example given in the 8th post in the thread would be handled by adding
	the following lines to the CvCustomEventManager constructor.  The RealFort,
	TechConquest, and CulturalDecay classes can remain unmodified.
	
		self.addEventHandler("unitMove", rf.onUnitMove)
		self.addEventHandler("improvementBuilt", rf.onImprovementBuilt)
		self.addEventHandler("techAcquired", rf.onTechAcquired)
		self.addEventHandler("cityAcquired", tc.onCityAcquired)
		self.addEventHandler("EndGameTurn", cd.onEndGameTurn)
		
	Note that the naming conventions for the event type strings vary from event
	to event.  Some use initial capitalization, some do not; some eliminate the
	"on___" prefix used in the event handler function name, some do not.  Look
	at the unmodified CvEventManager.py source code to determine the correct
	name for a particular event.
	
	Take care with event handlers that also extend CvEventManager.  Since
	this event manager handles invocation of the base class handler function,
	additional handlers should not also call the base class function themselves.

	"""

	def __init__(self, *args, **kwargs):
		super(CvEventManagerControlFile, self).__init__(*args, **kwargs)
		# map the initial EventHandlerMap values into the new data structure
		for eventType, eventHandler in self.EventHandlerMap.iteritems():
			self.setEventHandler(eventType, eventHandler)
		# --> INSERT EVENT HANDLER INITIALIZATION HERE <--

		print ">>>Begin Loading Event Managers"
	
		self.eventManager = list()
		for sz in CvPath.userActiveModDir, CvPath.installActiveModDir:
			if os.path.exists(sz):
				extraPaths = os.listdir(os.path.join(os.path.join(sz, "Assets"), "Python"))
				extraPaths.append("")
				for i in range(len(extraPaths)):
					if extraPaths[i] != "" and len(extraPaths[i]) > 0:
						eventManagerPath = os.path.join(os.path.join(os.path.join(sz, "Assets"), "Python"), extraPaths[i])
					else:
						eventManagerPath = os.path.join(os.path.join(sz, "Assets"), "Python")
	
					print ">>>Checking %s" %(eventManagerPath)
					if os.path.exists(eventManagerPath) and os.path.isdir(eventManagerPath):
						print ">>>___%s Exists" %(eventManagerPath)
						loadOrder = self.getLoadOrderFile(eventManagerPath)
						dir = []
						if loadOrder != None:
							pSpecialLoads = zConfigParser.parse(loadOrder).getNodeTree()
							dir = pSpecialLoads.getAll("Event Manager", "File")
							print ">>>___Using Load Order File"
						print ">>>______Checking these files %s" %(str(dir))
						for ii in range(len(dir)):
							szFile = dir[ii]
							print ">>>______Checking file %s" %(szFile)
							if os.path.splitext(szFile)[1].lower() in [".py", ".pyc"] and ( os.path.splitext(szFile)[0].lower() != "cveventmanagercontrolfile" or extraPaths[i] != "" ) and os.path.exists(os.path.join(eventManagerPath, szFile)):
								print ">>>_________%s is loadable" %(szFile)
								if loadOrder != None:
									print ">>>_________%s is an Event Manager file" %(szFile)
									self.importFile(os.path.splitext(szFile)[0])
								else:
									print ">>>_________%s is not an Event Manager file" %(szFile)
							else:
								print ">>>_________%s is not loadable" %(szFile)
					else:
						print ">>>___%s Does Not Exist" %(eventManagerPath)

	def importFile(self, szFileName):
		try:
			print ">>>Importing %s" %(szFileName)
			newEventManager = __import__(szFileName).EventManager(self)
			self.eventManager.append(newEventManager)
			print ">>>Imported %s" %(szFileName)
		except:
			print ">>>Failed to Import %s" %(szFileName)

	def getLoadOrderFile(self, szPath):
		dir = os.listdir(szPath)
		for i in range(len(dir)):
			if dir[i].lower().find("loadorder") > -1 and os.path.splitext(dir[i])[1].lower() == ".zcnfg":
				return os.path.join(szPath, dir[i])
		return None

	def addEventHandler(self, eventType, eventHandler):
		"""Adds a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.

		"""
		self.EventHandlerMap[eventType].append(eventHandler)

	def removeEventHandler(self, eventType, eventHandler):
		"""Removes a handler for the given event type.
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  It is an error if 
		the given handler is not found in the list of installed handlers.

		"""
		self.EventHandlerMap[eventType].remove(eventHandler)
	
	def setEventHandler(self, eventType, eventHandler):
		"""Removes all previously installed event handlers for the given 
		event type and installs a new handler .
		
		A list of supported event types can be found in the initialization 
		of EventHandlerMap in the CvEventManager class.  This method is 
		primarily useful for overriding, rather than extending, the default 
		event handler functionality.

		"""
		self.EventHandlerMap[eventType] = [eventHandler]

	def handleEvent(self, argsList):
		"""Handles events by calling all installed handlers."""
		self.origArgsList = argsList
		flagsIndex = len(argsList) - 6
		self.bDbg, self.bMultiPlayer, self.bAlt, self.bCtrl, self.bShift, self.bAllowCheats = argsList[flagsIndex:]
		eventType = argsList[0]
		return {
			"kbdEvent": self._handleConsumableEvent,
			"mouseEvent": self._handleConsumableEvent,
			"OnSave": self._handleOnSaveEvent,
			"OnLoad": self._handleOnLoadEvent
		}.get(eventType, self._handleDefaultEvent)(eventType, argsList[1:])

	def _handleDefaultEvent(self, eventType, argsList):
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				eventHandler(argsList[:len(argsList) - 6])

	def _handleConsumableEvent(self, eventType, argsList):
		"""Handles events that can be consumed by the handlers, such as
		keyboard or mouse events.
		
		If a handler returns non-zero, processing is terminated, and no 
		subsequent handlers are invoked.

		"""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = eventHandler(argsList[:len(argsList) - 6])
				if (result > 0):
					return result
		return 0

	# TODO: this probably needs to be more complex
	def _handleOnSaveEvent(self, eventType, argsList):
		"""Handles OnSave events by concatenating the results obtained
		from each handler to form an overall consolidated save string.

		"""
		result = ""
		if self.EventHandlerMap.has_key(eventType):
			for eventHandler in self.EventHandlerMap[eventType]:
				# the last 6 arguments are for internal use by handleEvent
				result = result + eventHandler(argsList[:len(argsList) - 6])
		return result

	# TODO: this probably needs to be more complex
	def _handleOnLoadEvent(self, eventType, argsList):
		"""Handles OnLoad events."""
		return self._handleDefaultEvent(eventType, argsList)