## Sid Meier's Civilization 4
## Copyright Firaxis Games 2005
##
## Implementaion of miscellaneous game functions

import CvUtil
from CvPythonExtensions import *
import CvEventInterface

import os
import CvPath
import zConfigParser

# globals
gc = CyGlobalContext()

class CvGameUtils:
	"Miscellaneous game functions"
	def __init__(self): 

		self.callbacks =	{
					"isVictoryTest"				: ([], ""),
					"isVictory"				: ([], ""),
					"isPlayerResearch"			: ([], "USE_IS_PLAYER_RESEARCH_CALLBACK"),
					"getExtraCost"				: ([], ""),
					"createBarbarianCities"			: ([], ""),
					"createBarbarianUnits"			: ([], ""),
					"skipResearchPopup"			: ([], ""),
					"showTechChooserButton"			: ([], ""),
					"getFirstRecommendedTech"		: ([], ""),
					"getSecondRecommendedTech"		: ([], ""),
					"canRazeCity"				: ([], ""),
					"canDeclareWar"				: ([], "USE_CAN_DECLARE_WAR_CALLBACK"),
					"skipProductionPopup"			: ([], ""),
					"showExamineCityButton"			: ([], ""),
					"getRecommendedUnit"			: ([], ""),
					"getRecommendedBuilding"		: ([], ""),
					"updateColoredPlots"			: ([], ""),
					"isActionRecommended"			: ([], ""),
					"unitCannotMoveInto"			: ([], "USE_UNIT_CANNOT_MOVE_INTO_CALLBACK"),
					"cannotHandleAction"			: ([], "USE_CANNOT_HANDLE_ACTION_CALLBACK"),
					"canBuild"				: ([], "USE_CAN_BUILD_CALLBACK"),
					"cannotFoundCity"			: ([], "USE_CANNOT_FOUND_CITY_CALLBACK"),
					"cannotSelectionListMove"		: ([], ""),
					"cannotSelectionListGameNetMessage"	: ([], ""),
					"cannotDoControl"			: ([], ""),
					"canResearch"				: ([], "USE_CAN_RESEARCH_CALLBACK"),
					"cannotResearch"			: ([], "USE_CANNOT_RESEARCH_CALLBACK"),
					"canDoCivic"				: ([], "USE_CAN_DO_CIVIC_CALLBACK"),
					"cannotDoCivic"				: ([], "USE_CANNOT_DO_CIVIC_CALLBACK"),
					"canTrain"				: ([], "USE_CAN_TRAIN_CALLBACK"),
					"cannotTrain"				: ([], "USE_CANNOT_TRAIN_CALLBACK"),
					"canConstruct"				: ([], "USE_CAN_CONSTRUCT_CALLBACK"),
					"cannotConstruct"			: ([], "USE_CANNOT_CONSTRUCT_CALLBACK"),
					"canCreate"				: ([], ""),
					"cannotCreate"				: ([], ""),
					"canMaintain"				: ([], ""),
					"cannotMaintain"			: ([], ""),
					"AI_chooseTech"				: ([], ""),
					"AI_chooseProduction"			: ([], ""),
					"AI_unitUpdate"				: ([], ""),
					"AI_doWar"				: ([], ""),
					"AI_doDiplo"				: ([], ""),
					"calculateScore"			: ([], ""),
					"doHolyCity"				: ([], ""),
					"doHolyCityTech"			: ([], ""),
					"doGold"				: ([], ""),
					"doResearch"				: ([], ""),
					"doGoody"				: ([], ""),
					"doGrowth"				: ([], ""),
					"doProduction"				: ([], ""),
					"doCulture"				: ([], ""),
					"doPlotCulture"				: ([], ""),
					"doReligion"				: ([], ""),
					"cannotSpreadReligion"			: ([], "USE_CANNOT_SPREAD_RELIGION_CALLBACK"),
					"doGreatPeople"				: ([], ""),
					"doMeltDown"				: ([], ""),
					"doReviveActivePlayer"			: ([], ""),
					"doPillageGold"				: ([], ""),
					"doCityCaptureGold"			: ([], ""),
					"citiesDestroyFeatures"			: ([], ""),
					"canFoundCitiesOnWater"			: ([], "USE_CAN_FOUND_CITIES_ON_WATER_CALLBACK"),
					"doCombat"				: ([], ""),
					"getConscriptUnitType"			: ([], ""),
					"getCityFoundValue"			: ([], "USE_GET_CITY_FOUND_VALUE_CALLBACK"),
					"canPickPlot"				: ([], ""),
					"getUnitCostMod"			: ([], "USE_GET_UNIT_COST_MOD_CALLBACK"),
					"getBuildingCostMod"			: ([], "USE_GET_BUILDING_COST_MOD_CALLBACK"),
					"canUpgradeAnywhere"			: ([], ""),
					"getWidgetHelp"				: ([], ""),
					"getUpgradePriceOverride"		: ([], ""),
					"getExperienceNeeded"			: ([], "")
					}

		print ">>>Begin Loading Game Util Files"
	
		self.gameUtils = list()
		for sz in CvPath.userActiveModDir, CvPath.installActiveModDir:
			if os.path.exists(sz):
				extraPaths = os.listdir(os.path.join(os.path.join(sz, "Assets"), "Python"))
				extraPaths.append("")		## Default Dir
				for i in range(len(extraPaths)):
					if extraPaths[i] != "" and len(extraPaths[i]) > 0:
						gameUtilsPath = os.path.join(os.path.join(os.path.join(sz, "Assets"), "Python"), extraPaths[i])
					else:
						gameUtilsPath = os.path.join(os.path.join(sz, "Assets"), "Python")
	
					print ">>>Checking %s" %(gameUtilsPath)
					if os.path.exists(gameUtilsPath) and os.path.isdir(gameUtilsPath):
						print ">>>___%s Exists" %(gameUtilsPath)
						loadOrder = self.getLoadOrderFile(gameUtilsPath)
						dir = []
						if loadOrder != None:
							pSpecialLoads = zConfigParser.parse(loadOrder).getNodeTree()
							dir = pSpecialLoads.getAll("Game Utils", "File")
							print ">>>___Using Load Order File"
						print ">>>______Checking these files %s" %(str(dir))
						for ii in range(len(dir)):
							szFile = dir[ii]
							print ">>>______Checking file %s" %(szFile)
							if os.path.splitext(szFile)[1].lower() in [".py", ".pyc"] and ( os.path.splitext(szFile)[0].lower() != "cvgameutils" or extraPaths[i] != "" ) and os.path.exists(os.path.join(gameUtilsPath, szFile)):
								print ">>>_________%s is loadable" %(szFile)
								if loadOrder != None:
									print ">>>_________%s is a GameUtils file" %(szFile)
									self.importFile(os.path.splitext(szFile)[0])
								else:
									print ">>>_________%s is not a GameUtils file" %(szFile)
							else:
								print ">>>_________%s is not loadable" %(szFile)
					else:
						print ">>>___%s Does Not Exist" %(gameUtilsPath)

	def importFile(self, szFileName):
		print ">>>Importing %s" %(szFileName)
		newGameUtil = __import__(szFileName).GameUtils(self)
		self.gameUtils.append(newGameUtil)
		print ">>>Imported %s" %(szFileName)

	def getLoadOrderFile(self, szPath):
		dir = os.listdir(szPath)
		for i in range(len(dir)):
			if dir[i].lower().find("loadorder") > -1 and os.path.splitext(dir[i])[1].lower() == ".zcnfg":
				return os.path.join(szPath, dir[i])
		return None

	def handleCallback(self, callback, argsList, defaultVal):
		for i in range(len(self.callbacks[callback][0])):
			if argsList == None:
				ret = self.callbacks[callback][0][i]()
			else:
				ret = self.callbacks[callback][0][i](argsList)

			if ret != defaultVal:
				return ret

		return defaultVal

	def addCallbackHandler(self, callback, handler):
		if self.callbacks.has_key(callback):
			self.callbacks[callback][0].append(handler)
			if self.callbacks[callback][1] != "":
				gc.setDefineINT(self.callbacks[callback][1], 1)
			print ">>>Created callback %s, now there are %d callbacks of this type" %(callback, len(self.callbacks[callback][0]))
	
	def isVictoryTest(self):
		if ( gc.getGame().getElapsedGameTurns() > 10 ):
			return self.handleCallback("isVictoryTest", None, True)
		else:
			return self.handleCallback("isVictoryTest", None, False)

	def isVictory(self, argsList):
		eVictory = argsList[0]
		return self.handleCallback("isVictory", argsList, True)

	def isPlayerResearch(self, argsList):
		ePlayer = argsList[0]
		return self.handleCallback("isPlayerResearch", argsList, True)

	def getExtraCost(self, argsList):
		ePlayer = argsList[0]
		return self.handleCallback("getExtraCost", argsList, 0)

	def createBarbarianCities(self):
		return self.handleCallback("createBarbarianCities", None, False)
		
	def createBarbarianUnits(self):
		return self.handleCallback("createBarbarianUnits", None, False)
		
	def skipResearchPopup(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("skipResearchPopup", argsList, False)
		
	def showTechChooserButton(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("showTechChooserButton", argsList, True)

	def getFirstRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("getFirstRecommendedTech", argsList, TechTypes.NO_TECH)

	def getSecondRecommendedTech(self,argsList):
		ePlayer = argsList[0]
		eFirstTech = argsList[1]
		return self.handleCallback("getSecondRecommendedTech", argsList, TechTypes.NO_TECH)
	
	def canRazeCity(self,argsList):
		iRazingPlayer, pCity = argsList
		return self.handleCallback("canRazeCity", argsList, True)
	
	def canDeclareWar(self,argsList):
		iAttackingTeam, iDefendingTeam = argsList
		return self.handleCallback("canDeclareWar", argsList, True)
	
	def skipProductionPopup(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("skipProductionPopup", argsList, False)
		
	def showExamineCityButton(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("showExamineCityButton", argsList, True)

	def getRecommendedUnit(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("getRecommendedUnit", argsList, UnitTypes.NO_UNIT)

	def getRecommendedBuilding(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("getRecommendedBuilding", argsList, BuildingTypes.NO_BUILDING)

	def updateColoredPlots(self):
		return self.handleCallback("updateColoredPlots", None, False)

	def isActionRecommended(self,argsList):
		pUnit = argsList[0]
		iAction = argsList[1]
		return self.handleCallback("isActionRecommended", argsList, False)

	def unitCannotMoveInto(self,argsList):
		ePlayer = argsList[0]		
		iUnitId = argsList[1]
		iPlotX = argsList[2]
		iPlotY = argsList[3]
		return self.handleCallback("unitCannotMoveInto", argsList, False)

	def cannotHandleAction(self,argsList):
		pPlot = argsList[0]
		iAction = argsList[1]
		bTestVisible = argsList[2]
		return self.handleCallback("cannotHandleAction", argsList, False)

	def canBuild(self,argsList):
		iX, iY, iBuild, iPlayer = argsList
		return self.handleCallback("canBuild", argsList, -1)	# Returning -1 means ignore; 0 means Build cannot be performed; 1 or greater means it can

	def cannotFoundCity(self,argsList):
		iPlayer, iPlotX, iPlotY = argsList
		return self.handleCallback("cannotFoundCity", argsList, False)

	def cannotSelectionListMove(self,argsList):
		pPlot = argsList[0]
		bAlt = argsList[1]
		bShift = argsList[2]
		bCtrl = argsList[3]
		return self.handleCallback("cannotSelectionListMove", argsList, False)

	def cannotSelectionListGameNetMessage(self,argsList):
		eMessage = argsList[0]
		iData2 = argsList[1]
		iData3 = argsList[2]
		iData4 = argsList[3]
		iFlags = argsList[4]
		bAlt = argsList[5]
		bShift = argsList[6]
		return self.handleCallback("cannotSelectionListGameNetMessage", argsList, False)

	def cannotDoControl(self,argsList):
		eControl = argsList[0]
		return self.handleCallback("cannotDoControl", argsList, False)

	def canResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		return self.handleCallback("canResearch", argsList, False)

	def cannotResearch(self,argsList):
		ePlayer = argsList[0]
		eTech = argsList[1]
		bTrade = argsList[2]
		return self.handleCallback("cannotResearch", argsList, False)

	def canDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		return self.handleCallback("canDoCivic", argsList, False)

	def cannotDoCivic(self,argsList):
		ePlayer = argsList[0]
		eCivic = argsList[1]
		return self.handleCallback("cannotDoCivic", argsList, False)
		
	def canTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		bIgnoreUpgrades = argsList[5]
		return self.handleCallback("canTrain", argsList, False)

	def cannotTrain(self,argsList):
		pCity = argsList[0]
		eUnit = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		bIgnoreUpgrades = argsList[5]
		return self.handleCallback("cannotTrain", argsList, False)

	def canConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		return self.handleCallback("canConstruct", argsList, False)

	def cannotConstruct(self,argsList):
		pCity = argsList[0]
		eBuilding = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		bIgnoreCost = argsList[4]
		return self.handleCallback("cannotConstruct", argsList, False)

	def canCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return self.handleCallback("canCreate", argsList, False)

	def cannotCreate(self,argsList):
		pCity = argsList[0]
		eProject = argsList[1]
		bContinue = argsList[2]
		bTestVisible = argsList[3]
		return self.handleCallback("cannotCreate", argsList, False)

	def canMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return self.handleCallback("canMaintain", argsList, False)

	def cannotMaintain(self,argsList):
		pCity = argsList[0]
		eProcess = argsList[1]
		bContinue = argsList[2]
		return self.handleCallback("cannotMaintain", argsList, False)

	def AI_chooseTech(self,argsList):
		ePlayer = argsList[0]
		bFree = argsList[1]
		return self.handleCallback("AI_chooseTech", argsList, TechTypes.NO_TECH)

	def AI_chooseProduction(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("AI_chooseProduction", argsList, False)

	def AI_unitUpdate(self,argsList):
		pUnit = argsList[0]
		return self.handleCallback("AI_unitUpdate", argsList, False)

	def AI_doWar(self,argsList):
		eTeam = argsList[0]
		return self.handleCallback("AI_doWar", argsList, False)

	def AI_doDiplo(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("AI_doDiplo", argsList, False)

	def calculateScore(self,argsList):
		ePlayer = argsList[0]
		bFinal = argsList[1]
		bVictory = argsList[2]
		
		iPopulationScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getPopScore(), gc.getGame().getInitPopulation(), gc.getGame().getMaxPopulation(), gc.getDefineINT("SCORE_POPULATION_FACTOR"), True, bFinal, bVictory)
		iLandScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getLandScore(), gc.getGame().getInitLand(), gc.getGame().getMaxLand(), gc.getDefineINT("SCORE_LAND_FACTOR"), True, bFinal, bVictory)
		iTechScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getTechScore(), gc.getGame().getInitTech(), gc.getGame().getMaxTech(), gc.getDefineINT("SCORE_TECH_FACTOR"), True, bFinal, bVictory)
		iWondersScore = CvUtil.getScoreComponent(gc.getPlayer(ePlayer).getWondersScore(), gc.getGame().getInitWonders(), gc.getGame().getMaxWonders(), gc.getDefineINT("SCORE_WONDER_FACTOR"), False, bFinal, bVictory)
		return self.handleCallback("calculateScore", argsList, int(iPopulationScore + iLandScore + iWondersScore + iTechScore))

	def doHolyCity(self):
		return self.handleCallback("doHolyCity", None, False)

	def doHolyCityTech(self,argsList):
		eTeam = argsList[0]
		ePlayer = argsList[1]
		eTech = argsList[2]
		bFirst = argsList[3]
		return self.handleCallback("doHolyCityTech", argsList, False)

	def doGold(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("doGold", argsList, False)

	def doResearch(self,argsList):
		ePlayer = argsList[0]
		return self.handleCallback("doResearch", argsList, False)

	def doGoody(self,argsList):
		ePlayer = argsList[0]
		pPlot = argsList[1]
		pUnit = argsList[2]
		return self.handleCallback("doGoody", argsList, False)

	def doGrowth(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doGrowth", argsList, False)

	def doProduction(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doProduction", argsList, False)

	def doCulture(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doCulture", argsList, False)

	def doPlotCulture(self,argsList):
		pCity = argsList[0]
		bUpdate = argsList[1]
		ePlayer = argsList[2]
		iCultureRate = argsList[3]
		return self.handleCallback("doPlotCulture", argsList, False)

	def doReligion(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doReligion", argsList, False)

	def cannotSpreadReligion(self,argsList):
		iOwner, iUnitID, iReligion, iX, iY = argsList[0]
		return self.handleCallback("cannotSpreadReligion", argsList, False)

	def doGreatPeople(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doGreatPeople", argsList, False)

	def doMeltdown(self,argsList):
		pCity = argsList[0]
		return self.handleCallback("doMeltDown", argsList, False)
	
	def doReviveActivePlayer(self,argsList):
		"allows you to perform an action after an AIAutoPlay"
		iPlayer = argsList[0]
		return self.handleCallback("doReviveActivePlayer", argsList, False)
	
	def doPillageGold(self, argsList):
		"controls the gold result of pillaging"
		pPlot = argsList[0]
		pUnit = argsList[1]
		
		iPillageGold = 0
		iPillageGold = CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 1")
		iPillageGold += CyGame().getSorenRandNum(gc.getImprovementInfo(pPlot.getImprovementType()).getPillageGold(), "Pillage Gold 2")

		iPillageGold += (pUnit.getPillageChange() * iPillageGold) / 100
		
		return self.handleCallback("doPillageGold", argsList, iPillageGold)
	
	def doCityCaptureGold(self, argsList):
		"controls the gold result of capturing a city"
		
		pOldCity = argsList[0]
		
		iCaptureGold = 0
		
		iCaptureGold += gc.getDefineINT("BASE_CAPTURE_GOLD")
		iCaptureGold += (pOldCity.getPopulation() * gc.getDefineINT("CAPTURE_GOLD_PER_POPULATION"))
		iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND1"), "Capture Gold 1")
		iCaptureGold += CyGame().getSorenRandNum(gc.getDefineINT("CAPTURE_GOLD_RAND2"), "Capture Gold 2")

		if (gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS") > 0):
			iCaptureGold *= cyIntRange((CyGame().getGameTurn() - pOldCity.getGameTurnAcquired()), 0, gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS"))
			iCaptureGold /= gc.getDefineINT("CAPTURE_GOLD_MAX_TURNS")
		
		return self.handleCallback("doCityCaptureGold", argsList, iCaptureGold)
	
	def citiesDestroyFeatures(self,argsList):
		iX, iY= argsList
		return self.handleCallback("citiesDestroyFeatures", argsList, True)
		
	def canFoundCitiesOnWater(self,argsList):
		iX, iY= argsList
		return self.handleCallback("canFoundCitiesOnWater", argsList, False)
		
	def doCombat(self,argsList):
		pSelectionGroup, pDestPlot = argsList
		return self.handleCallback("doCombat", argsList, False)

	def getConscriptUnitType(self, argsList):
		iPlayer = argsList[0]
		iConscriptUnitType = -1 #return this with the value of the UNIT TYPE you want to be conscripted, -1 uses default system
		
		return self.handleCallback("getConscriptUnitType", argsList, -1)

	def getCityFoundValue(self, argsList):
		iPlayer, iPlotX, iPlotY = argsList
		iFoundValue = -1 # Any value besides -1 will be used
		
		return self.handleCallback("getCityFoundValue", argsList, -1)
		
	def canPickPlot(self, argsList):
		pPlot = argsList[0]
		return self.handleCallback("canPickPlot", argsList, True)
		
	def getUnitCostMod(self, argsList):
		iPlayer, iUnit = argsList
		iCostMod = -1 # Any value > 0 will be used
		
		return self.handleCallback("getUnitCostMod", argsList, iCostMod)

	def getBuildingCostMod(self, argsList):
		iPlayer, iCityID, iBuilding = argsList
		pPlayer = gc.getPlayer(iPlayer)
		pCity = pPlayer.getCity(iCityID)
		
		iCostMod = -1 # Any value > 0 will be used
		
		return self.handleCallback("getBuildingCostMod", argsList, iCostMod)
		
	def canUpgradeAnywhere(self, argsList):
		pUnit = argsList
		
		bCanUpgradeAnywhere = 0
		
		return self.handleCallback("canUpgradeAnywhere", argsList, bCanUpgradeAnywhere)
		
	def getWidgetHelp(self, argsList):
		eWidgetType, iData1, iData2, bOption = argsList
		
		return self.handleCallback("getWidgetHelp", argsList, u"")
		
	def getUpgradePriceOverride(self, argsList):
		iPlayer, iUnitID, iUnitTypeUpgrade = argsList
		
		return self.handleCallback("getUpgradePriceOverride", argsList, -1)	# Any value 0 or above will be used
	
	def getExperienceNeeded(self, argsList):
		# use this function to set how much experience a unit needs
		iLevel, iOwner = argsList
		
		iExperienceNeeded = 0

		# regular epic game experience		
		iExperienceNeeded = iLevel * iLevel + 1

		iModifier = gc.getPlayer(iOwner).getLevelExperienceModifier()
		if (0 != iModifier):
			iExperienceNeeded += (iExperienceNeeded * iModifier + 99) / 100   # ROUND UP
			
		return self.handleCallback("getExperienceNeeded", argsList, iExperienceNeeded)