#################################################################################
#										#
#				   zConfig v1.00				#
#			Created	by Zebra 9 December 20th, 2007			#
#										#
#	This is a SPECIAL VERSION. It is designed for use with Sid Meier's	#
#	Civilization 4, and its respective expasion packs. You can contact	#
#	Zebra 9 at Civfanatics.com or E-Mail him at				#
#	Zebra9@MyChristianSite.com. This is intended only for use as a		#
#	simple data storage system. Zebra 9 is not liable for any damage  	#
#	or data lose incurred by usage of this script. If you use this,		#
#	please give me credit, thank you and enjoy. :-)				#
#										#
#################################################################################

import sys
import os
import re
import time
import CvPath

class typeListing:
	def __init__(self):
		self.NODE_BASE = 0
		self.NODE_DOCUMENT = 1
		self.NODE_TAG = 2
		self.NODE_VALUE = 3
		self.NODE_BODY = 4

	def getNodeTypeName(self, nodeType):
		"""Returns the string name of a node type"""
		if nodeType == self.NODE_BASE:
			return "Base"
		elif nodeType == self.NODE_DOCUMENT:
			return "Document"
		elif nodeType == self.NODE_TAG:
			return "Tag"
		elif nodeType == self.NODE_VALUE:
			return "Value"
		elif nodeType == self.NODE_BODY:
			return "Body"

types = typeListing()

class parser:
	def __init__(self, fileName, **info):
		cleanFilePath = info.get("cleanFilePath", None)
		self.__fileName = fileName
		self.__parsed = False
		self.__numLinesRead = 0

		self.__nodeTree = document(self)
		self.log = info.get("log", False)

		if info.get("parse", True):
			self.parse(cleanFilePath)

	def getFileName(self):
		"""Returns the name of the file that this parse object was created for"""
		return self.__fileName

	def parse(self, cleanFilePath = None):
		"""	Parses the file
			It takes 1 optional argument cleanFilePath:
			If you pass \"\" it will clean the file being parsed,
			this is good for debugging because you can see exactly how the parser parsed the file
			or you could pass a file path, this file will be over writen if it exists or created if it does not,
			this is usful for debugging as well because it creates a copy of the parsed file showing exactly how the parser parsed it
			or you could pass None (the default), this tells it to just parse the file and not to make a clean version"""

		if self.__fileName == None:
			raise fileNotFound(self.__fileName)
		elif not os.path.exists(self.__fileName):
			raise fileNotFound(self.__fileName)

		self.__nodeTree = document(self)
		zPrint(">>>Begin Parsing %s %s" %(self.getFileName(), getDateAndTime()), self.log)
		pFile = open(self.getFileName(), "r", -1)

		currentNode = self.__nodeTree.body
		currentValue = None

		for line in pFile:
			currentNode, currentValue = self._parse(line, currentNode, currentValue)
			self.__numLinesRead += 1

			if self.__numLinesRead % 100 == 0:
				zPrint("Read %d lines" %	self.__numLinesRead, self.log)

		pFile.close()
		zPrint(">>>Finished Parsing %s %s" %(self.getFileName(), getDateAndTime()), self.log)

		if cleanFilePath == "":
			zPrint(">>>Clean %s" %self.getFileName(), self.log)
			self.writeToFile(self.getFileName(), 1)
		elif cleanFilePath != None and os.path.exists(os.path.dirname(cleanFilePath)):
			zPrint(">>>Creat Clean version in %s" %cleanFilePath, self.log)
			self.writeToFile(cleanFilePath, 2)

		self.__parsed = True
		zPrint("Complete", self.log)

	def parseString(self, szString, cleanFilePath = None):
		"""	Parses the string szString
			It takes 1 optional argument cleanFilePath:
			You can pass a file path, this file will be over writen if it exists or created if it does not,
			this is usful for debugging because it creates a copy of the parsed string showing exactly how the parser parsed it
			or you could pass None (the default), this tells it to just parse the file and not to make a clean version"""
		self.__nodeTree = document(self)
		zPrint(">>>Begin Parsing %s %s" %("A String", getDateAndTime()), self.log)

		pLines = szString.split("\n")

		currentNode = self.__nodeTree.body
		currentValue = None

		for line in range(len(pLines)):
			currentNode, currentValue = self._parse(pLines[line], currentNode, currentValue)
			self.__numLinesRead += 1

			if self.__numLinesRead % 100 == 0:
				zPrint("Read %d lines" %	self.__numLinesRead, self.log)

		zPrint(">>>Finished Parsing %s %s" %("A String", getDateAndTime()), self.log)

		if cleanFilePath != None and os.path.exists(os.path.dirname(cleanFilePath)):
			zPrint(">>>Creat Clean version in %s" %cleanFilePath, self.log)
			self.writeToFile(cleanFilePath, 1)

		self.__parsed = True
		zPrint("Complete", self.log)

	def _parse(self, string, activeNode, activeValue):
 		if len(string) == 0 or string.startswith("#"):
			return activeNode, activeValue

		startEndTag = re.compile(r"^\s*\[[^\]\n\/]+\/\]")
		cleanStartEndTag1 = re.compile(r"^\s*\[")
		cleanStartEndTag2 = re.compile(r"\/\].*\n?")

		startTag = re.compile(r"^\s*\[[^\]\n\/]+\]")
		cleanStartTag1 = re.compile(r"^\s*\[")
		cleanStartTag2 = re.compile(r"\].*\n?")

		isEndTag = re.compile(r"^\s*\[\/[^\]\n\/]*\]")
		cleanEndTag1 = re.compile(r"^\s*\[\/")
		cleanEndTag2 = re.compile(r"\].*\n?")

		varNode = re.compile(r"^\s*[\w\s]+\s*(?<!\\)\=\s*.*")
		varSplit = re.compile(r"\s*\=\s*")

		splitLine = re.compile(r"(?<!\\)\;")

		entries = splitLine.split(string)

		zPrint("Checking \"%s\"" %(string), self.log, True)
		cleanLine = string.strip("\n")
		zPrint("Checking Clean Line \"%s\"" %(cleanLine), self.log, True)

		if len(startEndTag.findall(cleanLine)) > 0:
			tagName = cleanStartEndTag1.sub("", cleanLine)
			tagName = cleanStartEndTag2.sub("", tagName)

			zPrint("Creating Start/End Tag \"%s\"" %(tagName), self.log, True)

			newNode = tag(tagName)
			activeNode.appendNode(newNode)
			self.createStartEndTag(newNode)

			zPrint("Created Start/End Tag \"%s\"" %(tagName), self.log, True)

		elif len(startTag.findall(cleanLine)) > 0:
			tagName = cleanStartTag1.sub("", cleanLine)
			tagName = cleanStartTag2.sub("", tagName)

			zPrint("Creating Start Tag \"%s\"" %(tagName), self.log, True)

			newNode = tag(tagName)
			activeNode.appendNode(newNode)
			activeNode = newNode
			self.createStartTag(newNode)

			zPrint("Created Start Tag \"%s\"" %(tagName), self.log, True)
			zPrint("Current Node Has Been Changed To \"%s\"" %(activeNode.getName()), self.log, True)

		elif len(isEndTag.findall(cleanLine)) > 0:
			tagName = cleanEndTag1.sub("", cleanLine)
			tagName = cleanEndTag2.sub("", tagName)

			zPrint("Creating End Tag \"%s\"" %(tagName), self.log, True)

			if tagName == activeNode.getName() or tagName == "":
				zPrint("Current Node Is \"%s\"" %(activeNode.getName()), self.log, True)
				self.createEndTag(activeNode)
				activeNode = activeNode.getParent()
				activeValue = None

				zPrint("Created End Tag \"%s\"" %(tagName), self.log, True)
				zPrint("Current Node Has Been Changed To \"%s\"" %(activeNode.getName()), self.log, True)

		elif len(varNode.findall(cleanLine)) > 0:
			varData = varSplit.split(cleanLine, 1)

			if varData[0] == "":
				if activeValue != None:
					activeValue.setValue(activeValue.getValue() + "\n" + varData[1])
					self.extendValue(activeValue, varData[1])
			else:
				zPrint("Creating Value \"%s\" with \"%s\"" %(varData[0], varData[1]), self.log, True)

				newNode = value(varData[0], varData[1])
				activeNode.appendNode(newNode)
				activeValue = newNode
				self.createValue(activeValue, varData[1])

				zPrint("Created Value \"%s\" with \"%s\"" %(varData[0], varData[1]), self.log, True)
				zPrint("Current Node Is \"%s\"" %(activeNode.getName()), self.log, True)

		return activeNode, activeValue

	def getNodeTree(self):
		"""Returns the document node, the document node contains and gives some controls over the node tree"""
		return self.__nodeTree

	def writeToFile(self, fileName, isClean = 0):
		"""	Writes the parsed info to file fileName, isClean is used to tell it what type of header to place at the beginning of the file
			This function is used to create debug versions of parsed files. The parse function allows indirect usage of this function."""
		if os.path.splitext(fileName)[1] == "":
			fileName += ".zFig"
		if fileName == None:
			raise fileNotFound(fileName)

		f = open(fileName, "w")
		dateAndTime = getDateAndTime()
		f.write("#########################################################\n")
		if isClean == 1:
			f.write("#	Clean File Dynamicaly Created %s\n" %dateAndTime)
		elif isClean == 2:
			f.write("#	Clean File Dynamicaly Created %s\n" %dateAndTime)
			f.write("#	Master file was %s%s...%s%s\n" %(os.path.splitdrive(self.getFileName())[0], os.sep, os.sep, os.path.basename(self.getFileName())))
		elif isClean == 0:
			f.write("#	Version File Created %s\n" %dateAndTime)
			f.write("#	Master file was %s%s...%s%s\n" %(os.path.splitdrive(self.getFileName())[0], os.sep, os.sep, os.path.basename(self.getFileName())))

		f.write("#\n")
		f.write("#	zConfig developed by Zebra 9\n")
		f.write("#	You can contact me at\n")
		f.write("#		Civfanatics.com or\n")
		f.write("#		at zebra9@mychristiansite.com\n")
		f.write("#########################################################\n\n")

		def writer(node, isDoc, currentIndent = 0):
			children = node.getChildren()
			if not isDoc and not node.getType() == types.NODE_BODY:
				zPrint("Writing [%s] to %s%s...%s%s\n" %(node.getName(), os.path.splitdrive(f.name)[0], os.sep, os.sep, os.path.basename(f.name)), self.log)
				if len(children) > 0:
					f.write(creatIndent(currentIndent) + "[" + node.getName() + "]\n")
				else:
					f.write(creatIndent(currentIndent) + "[" + node.getName() + "/]\n")
				currentIndent = currentIndent + 1

			if len(children) > 0:
				for iChild in range(len(children)):
					child = children[iChild]
					if child.getType() == types.NODE_VALUE:
						splitVal = child.getValue().split("\n")
						first = True
						for i in range(len(splitVal)):
							if first:
								f.write(creatIndent(currentIndent) + child.getName() + " = " + splitVal[i] + "\n")
								first = False
							else:
								f.write(creatIndent(currentIndent) + " = " + splitVal[i] + "\n")
					elif child.getType() == types.NODE_TAG or child.getType() == types.NODE_BODY:
						writer(child, False, currentIndent)

				if not isDoc and not node.getType() == types.NODE_BODY:
					currentIndent = currentIndent - 1
					f.write(creatIndent(currentIndent) + "[/" + node.getName() + "]\n")

		def creatIndent(currentIndent):
			indent = ""
			for i in range(currentIndent):
				indent += "\t"
			return indent
		writer(self.getNodeTree(), True)
		f.close()

	def createValue(self, node, value):
		return

	def extendValue(self, node, value):
		return

	def createEndTag(self, node):
		return

	def createStartTag(self, node):
		return

	def createStartEndTag(self, node):
		return

class node:
	def __init__(self):
		self.__nodeType = types.NODE_BASE
		self.__nodeName = None

		self.__parentNode = None

		self.__document = None

	def _setDocument(self, newVal):
		"""Sets the document that this node belongs to"""
		self.__document = newVal

	def getDocument(self):
		"""Returns the document that this node belongs to"""
		return self.__document

	def getType(self):
		"""returns this node's type"""
		return self.__nodeType

	def _setNodeType(self, type):
		"""sets this node's type"""
		self.__nodeType = type

	def getName(self):
		"""returns this node's name"""
		return self.__nodeName

	def _setName(self, newName = None):
		"""sets this node's name"""
		if newName == "" or newName == None:
			raise failedToCreateChild("Incorrect Name, %s" %str(newName))
		else:
			self.__nodeName = newName.strip()

	def getParent(self):
		"""returns this nodes parent node"""
		return self.__parentNode

	def _setParent(self, parent):
		"""sets this nodes parent node"""
		self.__parentNode = parent

	def __repr__(self):
		return "%s node: %s" %(types.getNodeTypeName(self.getType()), self.getName())
			

class tag(node):
	def __init__(self, nodeName = None):
		node.__init__(self)

		self.__nextSibling = None
		self.__lastSibling = None

		self.__allChildNodesByType = dict()
		self.__allChildNodesByName = dict()

		self.__children = []

		if nodeName == None:
			raise nodeCreationFailed("No Name Given")

		self._setNodeType(types.NODE_TAG)
		self._setName(nodeName)

	def getNextSibling(self):
		"""returns this nodes next sibling"""
		return self.__nextSibling

	def getLastSibling(self):
		"""returns this nodes last sibling"""
		return self.__lastSibling

	def getNodesByName(self, szName):
		"""returns a list of child nodes with name szName"""
		name = szName.strip()
		if self.__allChildNodesByName.has_key(name):
			return self.__allChildNodesByName[name]
		return None

	def getNodesByType(self, type):
		"""returns a list of child nodes with type type"""
		if self.__allNodesByType.has_key(type):
			return self.__allChildNodesByType[type]
		return None

	def getFirstType(self, type):
		"""returns the first child node with type type"""
		if self.__allChildNodesByType.has_key(type) and len(self.__allChildNodesByType[type]) > 0:
			return self.__allChildNodesByType[type][0]
		return None

	def getFirstName(self, szName):
		"""returns the first child node with name szName"""
		name = szName.strip()
		if self.__allChildNodesByName.has_key(name):
			if len(self.__allChildNodesByName[name]) > 0:
				return self.__allChildNodesByName[name][0]
		return None

	def walk(self, func, walkAll = False):
		"""walks through this nodes children and every time it finds a new node it passes it to function func, can also walk every node that is below this one in the node tree (ie. all children and their children and their children ect.) when walkAll is true (defaults to false)"""
		children = self.getChildren()

		for iChild in range(len(children)):
			func(children[iChild])

			if children[iChild].getType() == types.NODE_TAG and walkAll:
				children[iChild].walk(func, walkAll)

	def getAll(self, parentName, childName, outFormat = None):
		"""returns a list containing the value of a value node with name childName who is a child of a node named parentNode who is intern a child of this node. The return values can also be formated by passing a formater function (ie. str, int, complex, ect.)"""
		ret = []
		parentNodes = self.getNodesByName(parentName)
		if parentNodes == None:
			return ret
		for i in range(len(parentNodes)):
			if parentNodes[i].getType() in [types.NODE_TAG, types.NODE_DOCUMENT]:
				childNodes = parentNodes[i].getNodesByName(childName)
				if childNodes == None:
					continue
				for ii in range(len(childNodes)):
					if childNodes[ii].getType() == types.NODE_VALUE:
						if outFormat != None:
							ret.append(outFormat(childNodes[ii]))
						else:
							ret.append(childNodes[ii].getValue())
		return ret

	def get(self, parentName, childName, outFormat = None):
		"""returns the value of a value node with name childName who is a child of a node named parentNode who is intern a child of this node. The return values can also be formated by passing a formater function (ie. str, int, complex, ect.)"""
		parentNodes = self.getNodesByName(parentName)
		if parentNodes == None:
			return None
		for i in range(len(parentNodes)):
			if parentNodes[i].getType() in [types.NODE_TAG, types.NODE_DOCUMENT]:
				childNodes = parentNodes[i].getNodesByName(childName)
				if childNodes == None:
					continue
				for ii in range(len(childNodes)):
					if childNodes[ii].getType() == types.NODE_VALUE:
						if outFormat != None:
							return outFormat(childNodes[ii].getValue())
						else:
							return childNodes[ii]

	def getChildren(self):
		"""returns a list of this node's children"""
		return self.__children

	def appendNode(self, childNode = None):
		"""adds a node to this node's child list"""
		if childNode == None:
			raise failedToCreateChild("No Child Given")
		else:
			lastChild = self.getLastChild()
			if lastChild != None:
				lastChild.__nextSibling = childNode
			childNode._setParent(self)
			childNode.__lastSibling = lastChild

			childNode._setDocument(self.getDocument())

			self.getDocument()._addNodeByName(childNode)
			self.getDocument()._addNodeByType(childNode)

			if self.getType() != types.NODE_DOCUMENT:
				self._addChildNodeByName(childNode)
				self._addChildNodeByType(childNode)

			self.__children.append(childNode)

	def getLastChild(self):
		"""returns this nodes last child (ie. the newest child)"""
		children = self.getChildren()
		if len(children) > 0:
			return children[len(children) - 1]
		else:
			return None

	def getFirstChild(self):
		"""returns this nodes first child (ie. the oldest child)"""
		children = self.getChildren()
		if len(children) > 0:
			return self.getChildren()[0]
		else:
			return None

	def _addChildNodeByName(self, node):
		"""adds child node to the list of children based on name. PRIVATE DON"T CALL! Automatically called by appendNode!"""
		if not self.__allChildNodesByName.has_key(node.getName()):
			self.__allChildNodesByName[node.getName()] = list()
		self.__allChildNodesByName[node.getName()].append(node)

	def _addChildNodeByType(self, node):
		"""adds child node to the list of children based on type. PRIVATE DON"T CALL! Automatically called by appendNode!"""
		if not self.__allChildNodesByType.has_key(node.getType()):
			self.__allChildNodesByType[node.getType()] = list()
		self.__allChildNodesByType[node.getType()].append(node)

	def __len__(self):
		return len(self.__children)

	def __iter__(self):
		return zIter(range(len(self)))

	def getNameIter(self):
		"""returns an iterator that will allow iteration over the node names"""
		return iter(self.__allChildNodesByName)

	def __getitem__(self, key):
		if isinstance(key, int):
			if len(self) <= key:
				raise IndexError, "%d is out of range, please use a number from 0 to %d" %(key, len(self)-1)
			return self.__children[key]
		elif isinstance(key, str):
			if self.__allChildNodesByName.has_key(key):
				return self.__allChildNodesByName[key]
			else:
				raise KeyError("%s was not found" %key)
		else:
			raise KeyError, "%s was not found" %str(key)

	def __setitem__(self, key, value):
		if self[key].getType() == types.NODE_VALUE:
			self[key].setValue(value)
		else:
			raise zUnMutable, "%s is not changable." %str(key)

	def __delitem__(self, key):
		if isinstance(key, int):
			if len(self) <= key:
				raise IndexError, "%d is out of range, please use a number from 0 to %d" %(key, len(self)-1)
			del self.__children[key]
		elif isinstance(key, str):
			if self.__allChildNodesByName.has_key(key):
				del self.__allChildNodesByName[key]
			else:
				raise KeyError, "%s was not found" %key
		else:
			raise KeyError, "%s was not found" %str(key)

	def __contains__(self, key):
		if self.__allChildNodesByName.has_key(key):
			return True
		else:
			return False

class body(tag):
	def __init__(self):
		tag.__init__(self, "body")
		self._setNodeType(types.NODE_BODY)

class document(tag):
	def __init__(self, parser):
		tag.__init__(self, "document")

		self.__parentNode = self

		self.__allNodesByType = dict()
		self.__allNodesByName = dict()

		self._setNodeType(types.NODE_DOCUMENT)
		self._setDocument(self)

		self.body = None
		self._createBody()

		self.parser = parser

	def getParser(self):
		return self.parser

	def _createBody(self):
		"""creates the main body node"""
		newNode = body()
		self.appendNode(newNode)
		self.body = newNode

	def getNodesByName(self, szName):
		"""returns a list containing all nodes with name szName (not just child nodes, if you want the top level nodes use the getNodesByName function in the body node)"""
		name = szName.strip()
		if self.__allNodesByName.has_key(name):
			return self.__allNodesByName[name]
		return None

	def getNodesByType(self, type):
		"""returns a list containing all nodes of type type (not just child nodes, if you want the top level nodes use the getNodesByName function in the body node)"""
		if self.__allNodesByType.has_key(type):
			return self.__allNodesByType[type]
		return None

	def getFirstType(self, type):
		"""returns the first node of type type (ie. the first node created of this type)"""
		if self.__allNodesByType.has_key(type) and len(self.__allNodesByType[type]) > 0:
			return self.__allNodesByType[type][0]
		return None

	def getFirstName(self, szName):
		"""returns the first node with name szName (ie. the first node created of this name)"""
		name = szName.strip()
		if self.__allNodesByName.has_key(name) and len(self.__allNodesByName[name]) > 0:
			return self.__allNodesByName[name][0]
		return None

	def _addNodeByName(self, node):
		"""adds child node to the list of children based on name. PRIVATE DON"T CALL! Automatically called by appendNode!"""
		if not self.__allNodesByName.has_key(node.getName()):
			self.__allNodesByName[node.getName()] = list()
		self.__allNodesByName[node.getName()].append(node)

	def _addNodeByType(self, node):
		"""adds child node to the list of children based on type. PRIVATE DON"T CALL! Automatically called by appendNode!"""
		if not self.__allNodesByType.has_key(node.getType()):
			self.__allNodesByType[node.getType()] = list()
		self.__allNodesByType[node.getType()].append(node)

	def walk(self, func, walkAll = False):
		"""walks through this nodes children and every time it finds a new node it passes it to function func, can also walk every node that is below this one in the node tree (ie. all children and their children and their children ect.) when walkAll is true (defaults to false)"""
		children = self.getChildren()

		for iChild in range(len(children)):
			func(children[iChild])

			if children[iChild].getType() == types.NODE_TAG and walkAll:
				children[iChild].walk(func, walkAll)

	def getAll(self, parentName, childName, outFormat = None):
		"""returns a list containing the value of a value node with name childName who is a child of a node named parentNode who is intern a child of this node. The return values can also be formated by passing a formater function (ie. str, int, complex, ect.)"""
		ret = []
		parentNodes = self.getNodesByName(parentName)
		if parentNodes == None:
			return ret
		for i in range(len(parentNodes)):
			if parentNodes[i].getType() in [types.NODE_TAG, types.NODE_DOCUMENT]:
				childNodes = parentNodes[i].getNodesByName(childName)
				if childNodes == None:
					continue
				for ii in range(len(childNodes)):
					if childNodes[ii].getType() == types.NODE_VALUE:
						if outFormat != None:
							ret.append(outFormat(childNodes[ii]))
						else:
							ret.append(childNodes[ii].getValue())
		return ret

	def get(self, parentName, childName, outFormat = None):
		"""returns the value of a value node with name childName who is a child of a node named parentNode who is intern a child of this node. The return values can also be formated by passing a formater function (ie. str, int, complex, ect.)"""
		parentNodes = self.getNodesByName(parentName)
		if parentNodes == None:
			return None
		for i in range(len(parentNodes)):
			if parentNodes[i].getType() in [types.NODE_TAG, types.NODE_DOCUMENT]:
				childNodes = parentNodes[i].getNodesByName(childName)
				if childNodes == None:
					continue
				for ii in range(len(childNodes)):
					if childNodes[ii].getType() == types.NODE_VALUE:
						if outFormat != None:
							return outFormat(childNodes[ii].getValue())
						else:
							return childNodes[ii]

class value(tag):
	def __init__(self, nodeName = None, value = None):
		node.__init__(self)

		self.__value = None

		if nodeName == None:
			raise nodeCreationFailed("No Name Given")
		if value == None:
			raise nodeCreationFailed("No Value Given")

		self._setNodeType(types.NODE_VALUE)
		self._setName(nodeName)
		self.setValue(value)

	def __str__(self):
		return self.getValue()
	def __int__(self):
		try:
			return self.getIntValue()
		except:
			return -1
	def __float__(self):
		try:
			return self.getFloatValue()
		except:
			return float(-1)
	def __complex__(self):
		try:
			return self.getComplexValue()
		except:
			return complex(-1)
	def __long__(self):
		try:
			return self.getLongValue()
		except:
			return long(-1)
	def __nonzero__(self):
		try:
			return self.getBoolValue()
		except:
			return False

	def getValue(self):
		"""returns this nodes value as a string"""
		return self.__value

	def getIntValue(self, defaultVal = None):
		"""returns this nodes value as an int, can pass a default"""
		try:
			return int(self.getValue())
		except:
			if defaultVal != None:
				try:
					return int(self.getFloatValue())
				except:
					return defaultVal
			else:
				return int(self.getFloatValue())
				

	def getFloatValue(self, defaultVal = None):
		"""returns this nodes value as a float, can pass a default"""
		if defaultVal != None:
			try:
				return float(self.getValue())
			except:
				return defaultVal
		else:
			return float(self.getValue())

	def getComplexValue(self, defaultVal = None):
		"""returns this nodes value as a complex number, can pass a default"""
		try:
			return complex(self.getValue())
		except:
			if defaultVal != None:
				try:
					return complex(self.getFloatValue())
				except:
					return defaultVal
			else:
				return complex(self.getFloatValue())

	def getLongValue(self, defaultVal = None):
		"""returns this nodes value as a long int, can pass a default"""
		try:
			return long(self.getValue())
		except:
			if defaultVal != None:
				try:
					return long(self.getFloatValue())
				except:
					return defaultVal
			else:
				return long(self.getFloatValue())

	def getBoolValue(self, defaultVal = None):
		"""returns this nodes value as a boolean (true or false), can pass a default"""
		if self.getValue().lower().find("no") > -1 or self.getValue().lower().find("false") > -1:
			return False
		elif self.getValue().lower().find("yes") > -1 or self.getValue().lower().find("true") > -1:
			return True
		elif defaultVal != None:
			try:
				return bool(self.getValue())
			except:
				return defaultVal
		else:
			return bool(self.getValue())

	def setValue(self, newVal):
		"""sets this nodes value"""
		self.__value = str(newVal)

	def __repr__(self):
		return "%s node: %s, value is %s" %(types.getNodeTypeName(self.getType()), self.getName(), self.getValue())

# exception classes
class Error(Exception):
    """Base class for ConfigParser exceptions."""

    def __init__(self, msg=''):
        self.message = msg
        Exception.__init__(self, msg)

    def __repr__(self):
        return self.message

    __str__ = __repr__

class fileNotFound(Error):
    """Raised when the file is not found."""

    def __init__(self, fileName):
        Error.__init__(self, 'The file %s was not found.' % (str(fileName)))
        self.fileName = fileName

class nodeCreationFailed(Error):
    """Raised when the file is not found."""

    def __init__(self, MSG):
        Error.__init__(self, 'Node creation failed: %s' % (str(MSG)))
        self.MSG = MSG

class failedToCreateChild(Error):
    """Raised when the file is not found."""

    def __init__(self, MSG):
        Error.__init__(self, 'Failed to create child: %s' % (str(MSG)))
        self.MSG = MSG

class zUnMutable(Error):
    """Raised when the file is not found."""

    def __init__(self, MSG):
        Error.__init__(self, 'Unchangable: %s' % (str(MSG)))
        self.MSG = MSG

class zIter:
	def __init__(self, items):
		self.items = items
		self.count = 0

	def next(self):
                if self.count == len(self.items):
                    raise StopIteration
		item = self.items[self.count]
		self.count += 1
		return item

cwd = CvPath.userActiveModDir
if not os.path.exists(cwd):
	cwd = CvPath.installActiveModDir
if not os.path.exists(cwd):
	cwd = os.getcwd()

cwd = os.path.join(cwd, "MOD LOGs")

printFile = None
parserFile = None
def zPrint(szText, log, isParser = False):
	global printFile
	global parserFile
	if not isParser:
		if printFile == None and log:
			if not os.path.exists(cwd):
				os.mkdir(cwd)
			printFile = open(os.path.join(cwd, "zPrint.txt"), "w", 1)
		if log:
			printFile.write(szText)
			printFile.write("\n")
			print "zPrint '%s'" %szText
	else:
		if parserFile == None and log:
			if not os.path.exists(cwd):
				os.mkdir(cwd)
			parserFile = open(os.path.join(cwd, "zParser.txt"), "w", 1)
		if log:
			parserFile.write(szText)
			parserFile.write("\n")
			print "zParser '%s'" %szText

def getDateAndTime():
	date = time.localtime()
	month = str(date[1])
	day = str(date[2])
	year = str(date[0])
	hour = date[3]
	minute = date[4]
	second = date[5]
	ampm = "AM"

	if hour == 0:
		hour = "12"
	elif hour > 12:
		hour = str(hour - 12)
		ampm = "PM"
	else:
		hour = str(hour)

	if minute < 10:
		minute = "0" + str(minute)
	else:
		minute = str(minute)

	if second < 10:
		second = "0" + str(second)
	else:
		second = str(second)

	dateAndTime = "%s/%s/%s, %s:%s:%s %s" %(month, day, year, hour, minute, second, ampm)
	return dateAndTime

if __name__ == '__main__':
	raw_input("Must be loaded as module")